#include <riter/RITERAngle.hh>

extern "C"  int cerenkov_kot_(float *,float *,float *,float *,float *,float *,float *);

int  RITERAngle::Calculate(RITERHits &f, RITERTracks  &t, float &th_cer, float &fi_cer, int &flag)
  
//**** cherenkov angle calculation (for known hit-particle)
  // rf    ! photon hit (virtual detector position)
  // TH_CER, FI_CER  ! cerenkov angle (result)
  // returned value =OK, if result exists 
{ 
  static float rmax2=geo->rmax2;
  static float Radius[2]={geo->Radius(0),geo->Radius(1)};
  static RITERVec CoC[2]={ geo->CoC(0) , geo->CoC(1) };
  static int corrections_on=geo->IsCorrections();
  register float path = 0;

  register int k0=  (f.rf.y()>0) ? 0 : 1;     //! detector: k0=0 up, k0=1 dn

  flag = 0;
  // coarse limitation
  register float radij2=(f.rf-t.trc[k0]).mag2();
  if ((radij2 > rmax2) ) return 1;        
  
  if (!t.Likelihood()->IsInside2(3,radij2/sqr(0.5*Radius[k0]))) return 2;
  

  register RITERVec dirf, dirch;  
  
  // accurate cerenkov angle calculation    
  if ( cerenkov_kot(t.r0,t.dir,f.rf,CoC[k0],Radius[k0],dirf,dirch) !=RITER_OK ) return 3;
  
  /* M.staric Fortran r.
cout <<"RAngle::"<< t.r0 << t.dir<< f.rf<< CoC[k0]<< Radius[k0] << endl;
   if ( cerenkov_kot_((float *)&t.r0,(float *)&t.dir,(float *)&f.rf,(float *)&CoC[k0],(float *)&Radius[k0],(float *)&dirf,(float *)&dirch) == 0) return 3;

   */

  th_cer=acos(dirch.z());
  

  register RITERVec  r1,r2; 
  
  if (!SphereIntersection(t.r,dirf,Radius[k0],CoC[k0],r1,path))return 5;
  
  if (k0) { 
    if ( !((!(t.wm))||(r1.y()<0))  ) return 11;
  } else { 
    if ( !(( (t.wm))||(r1.y()>0))  ) return 12;
  }


  if (corrections_on) {
    
    // corrections for misaligned mirrors
    
    SphereIntersection(t.r0,dirf,Radius[k0],CoC[k0],r1,path); 
    register int kk   = (r1.z()<0 ) ? 1 : 0;     
    register int isfm = (kk==k0) ? geo->Which_SFM(r1) : t.id_sfm[k0];
    if (isfm >= 0) {
      r2 =f.rf-r1;
      register int iplm= geo->Which_PLM(r1,r2,k0,path); 
      if (iplm >= 0) {    
	fi_cer=atan2(dirch.y(),dirch.x());
	th_cer=geo->Correct_thc( isfm, iplm, th_cer, fi_cer );
      }
    }
  }
  
  if ( t.Likelihood()->IsInside(2,th_cer) ) flag = 1; else flag = -1;
  return RITER_OK;  
};
//-------------------------------------------------------------      

int  RITERAngle::CalculateFull( RITERHits &f, RITERTracks  &t, 
				float &th_cer, float &fi_cer,
				int &isfm, int &iplm, int &flag)
  

{ 

  cout << "RITERAngle::CalculateFull Not implemented\n";
  return RITER_ERROR;
};
//-------------------------------------------------------------
int  RITERAngle::cerenkov_kot( RITERVec &r0, RITERVec &dir, 
			       RITERVec &rh, RITERVec rc,
			       float r_mi, RITERVec &dirf,RITERVec &dirch)

// calculation of cerenkov angle from photon hit on detector and part. track
// linear approximation resi_sin
  /*
    r0                 // point of photon emission (mid point on part. track)
    dir                // particle direction (unit vector)
    rh                 // photon hit on detector
    rc 
    r_mi               // center of curvature of mirror

    th_cer             // result if OK
    dirf               // result (photon direction vector) if OK
    return             // returned value OK
  */
{ 
  register RITERVec r1, r2, rr; 
  register float  th, fi;

  r1=r0-rc;
  r2=rh-rc;
  
  register float r1n = r1.mag();	
  register float r2n = r2.mag();	
  register float alfa=(r1*r2)/r1n/r2n;
  alfa=max(min(alfa,1.),-1.);
  alfa=acos(alfa);
  register float a   = r1n/r_mi;
  register float b   = r2n/r1n;
  if (  resi_sin(a,b,alfa,th,fi)!=RITER_OK) return RITER_ERROR;
  register float s_alfa = sin(alfa);
  register float c1     = r_mi/r1n*sin(alfa-fi)/s_alfa;
  register float c2     = r_mi/r2n*sin(fi)/s_alfa;
     
  rr=(c1-1.)*r1+c2*r2;
  dirf  = rr*(1/rr.mag());   
  dirch=transform_from_fixed(dir,dirf);
  return RITER_OK;
};
//-------------------------------------------------------------      

int  RITERAngle::resi_sin(float a,float b,float c,float &x,float &y)
  
  // find solution of equations:	sin(x) = a*sin(x+y)
  //		        		sin(x+y)=b*sin(c+x-y)
  // approximative solution (second order)
  
  //input : a,b,c;                // coeficients
  //output:  x,y;                 // result
  //flag  :   ok;                 // if all ok
{      
  float abb=1.-2*a*b+b;
  if(abb==0) {
    return RITER_ERROR;
  }
  
  x=a*b*c/abb;
  y=(1.-a)*b*c/abb;
  
  register float sx=sin(x);
  register float cx=cos(x);
  register float sxy=sin(x+y);
  register float cxy=cos(x+y);
  register float scxy=sin(c+x-y);
  register float ccxy=cos(c+x-y);
  
  register float a11=cx-a*cxy;
  register float a12=-a*cxy;
  register float a21=cxy-b*ccxy;
  register float a22=cxy+b*ccxy;

  register float d=a11*a22-a12*a21;
  if(d==0) return RITER_ERROR;
  

  register float b1=a*sxy-sx;
  register float b2=b*scxy-sxy;
  
  x=x+(b1*a22-b2*a12)/d;
  y=y+(b2*a11-b1*a21)/d;
  return RITER_OK;
};
//-------------------------------------------------------------      	      



