#include <riter/RITERMain.hh>

RITERGeometry *RITER_Geometry ;

int IsInside(float x,float y) 
{
  static float th_0=RITER_Geometry->th_0; 
  static float fi_0=RITER_Geometry->fi_0;
  float fx =fabs(x);
  float fy =fabs(y);

  if  (( fy > th_0 ) ||  (fx > fi_0)  )  return 0;
  if  (th_0-fx*0.3 < fy) return 0;
  return 1;
}

RITERGeometry::RITERGeometry()
  :  RITERSphericalmirror(), 
     RITERPlanarmirror(), 
     RITERSupermodule(),
     RITERBeamshroud()
{
  x_cut      = 0;
  y_cut      = 0;
  z_cut      = 4500;
  z_magnet   = 4500;
  float d_pad= 9;  
  sig_sfa    = .16e-3;
  sigthc     = 0.000450;
  an_zero    = 42.00; // N_0  detector response parameter N=N_O Z^2 L sin(th)^2

  z_rich_min = 8600;       // where to get RICH min point
  z_rich_max = 11600;      // where to get RICH end point
  th_0       = 0.145;      // maximum acceptance  y direction
  fi_0       = 0.195;      // maximum acceptance  x  direction
  acc_lens   = 0.157;

#ifndef NO_ARTE
  float radius=getRadius(ArtePointer<EVHD>(1)->run,1);
#else 
  float radius=0.054;
#endif

  ref_ind=1/cos(radius);
  cout << " RITER RITERGeo: Radius for beta=1 particles: " << radius << endl;
  cout << " RITER RITERGeo: Refractive index= " << ref_ind << endl;

  Kuip();

  p_min    = .511E-3/sqrt((ref_ind-1.)*(ref_ind+1.));

  an_zero  = an_zero/10.;
  float f  = r_mi[0]/2;
  sig_pad  = d_pad/f/sqrt(12.); 
  sigma[0] = sqrt(sig_pad*sig_pad+sig_sfa*sig_sfa+sigthc*sigthc);
  sigma[1] = sqrt(4*sig_pad*sig_pad+sig_sfa*sig_sfa+sigthc*sigthc);

  cout << " RITER RITERGeo: Photon sigma= m16-" << sigma[0] <<"m4 "<< sigma[1] <<endl;

  f_pad    = f/d_pad;
  float thc= acos(1./ref_ind);
  rmax2    = sqr(1.4*thc*f);
  // square of minimum radius
  rmin2    = 0; 
  ReadCorrections();
#ifdef RITER_DEBUG
  Print();
#endif
}

int RITERGeometry::Kuip(){
  
  if  (RITERrefind>1) {
    cout << " RITER RITERGeo: USER request RITER/REFRINDEX "<< endl;
    ref_ind=RITERrefind;
    cout << " RITER RITERGeo: Refractive index changed to = " << ref_ind << endl;
  } 
  if  (RITERradius>0) {
    cout << " RITER RITERGeo: USER request RITER/RADIUS => "<<  RITERradius << endl;
    ref_ind=1/cos(RITERradius);
    cout << " RITER RITERGeo: Refractive index changed to = " << ref_ind << endl;
  } 
  return RITER_OK;
}

int RITERGeometry::ReadCorrections(){

  char configdir[100];
  char* RITERconfigdir=getenv("RITER_CONFIGDIR");
  if (RITERconfigdir) {
    strcpy(configdir,RITERconfigdir);
    if (configdir[strlen(configdir)-1]!='/') strcat(configdir,"/");
  }
  else {
    char *hb=getenv("HB");
    if (!hb) {
      fprintf(stderr,"RITERINI:  $HB is not defined.\n");
      fprintf(stderr,"RITERINI: using afs settings \n");
      strcpy(hb,"/afs/desy.de/group/hera-b/ARTE/");
    }
    strcpy(configdir,hb);
    strcat(configdir,"/geo/RICH/");
  }
  char fmirror_corr[200];
  sprintf(fmirror_corr,"%s/Riter_mirror_correction.dat",configdir);              
  FILE *fm_corr= fopen(fmirror_corr,"r");
  char line[80],name[80];
  int nmi=0;
  if (fm_corr != NULL) {
    while (fgets(line,80,fm_corr)!=NULL) {
      if (line[0]!='#') {
	int id=0;
	float x=0,y=0,ex=0,ey=0;
	if (sscanf(line,"%s%d%f%f%f%f",&name[0],&id,&x,&y,&ex,&ey)==6) {
	  if (strstr(name,"S")!=NULL) {
	    dx_sph[id]=x/1000.*2.;
	    dy_sph[id]=y/1000.*2.;
	    nmi++;	
	  }
	  if (strstr(name,"P")!=NULL) {
	    dx_plm[id]=x/1000.*2.;
	    dy_plm[id]=y/1000.*2.;
	    nmi++;
	  }
	}
      }
    }
    fclose(fm_corr);     
    cout << nmi <<  " corrections for mirrors read" << endl;
    if (nmi)
      is_correction=1;
    else 
      is_correction=0;
  } else {
     is_correction=0;
  }
  
  cout << "RITERGeometry initialised " <<endl;	
  return RITER_OK;
};
// ------------------------------------------------------------------------

int RITERGeometry::Print() {

  cout << "Init constants=========" <<endl;

  PrintSphericalmirror();
  PrintPlanarmirror();
  PrintSupermodule();

  printf("z_rich_min:%f\n",z_rich_min);
  printf("z_rich_max:%f\n",z_rich_max);
  printf("an_zero:%f\n", an_zero);

  printf("ref_ind:%f\n",  ref_ind);
  printf("th_0:%f\n",  th_0);
  printf("fi_0:%f\n",  fi_0);
  printf("sig_sfa:%f\n",  sig_sfa);
  printf("sigthc:%f\n", sigthc); 
  printf("r_mi:%f\n",  r_mi[0]);
  printf("acc_lens:%f\n", acc_lens);
  cout <<"rp_md: "<<   rp_md << endl;
  cout <<"rp_mu: "<<   rp_mu << endl;
  cout <<"ezp_md: "<<  ezp_md << endl;
  cout <<"ezp_mu: "<<  ezp_mu << endl;
  cout <<"rmax2: "<<  rmax2  << endl;
  cout <<"rmax: "<<  sqrt(rmax2)  << endl;
    
  

return RITER_OK;

};
/* --------------------------------------------------------------*/

int RITERGeometry::src2det(int ism, int ir, int ic , int pmch , int type, 
	    int &sm, int &x0, int &y0  ) 
{
  // translation from Reinhard system to ljubljana one
  // on input :
  // supermodule     ism 
  // row (0..31)     ir
  // column (0..11)  ic
  // on output  
  // supermodule     sm 
  // x coordinate m16 pad units  x0
  // y coordinate m16 pad units  y0

  if (ism >6) {
    ism=20-ism;
    ic=11-ic;
    ir=31-ir;
  } 
 
  int ppx=0;
  int ppy=0;
  switch (type) {
  case 16: 
   ppx=   4*(ic%2)+3-(pmch%4);
   ppy=7-(4*(ir%2)+3-(pmch/4));
      break;
  case 4:
    ppx=2*(   2*(ic%2)+1-(pmch%2));
    ppy=2*(3-(2*(ir%2)+1-(pmch/2)));
    break;
  }
  x0=8*(ic/2)+ppx;
  y0=8*(ir/2)+ppy;
  sm=ism;
  return RITER_OK;
}
//----------------------------------------------------------------      
void RITERGeometry::to_xyz(int ism,int ir,int ic,int nn,RITERVec &r,float &sig)
// detector from planar supermodules
{ 
  float a=(ic+nn/2.)/48;
  float b=(ir+nn/2.)/128;
  r=r0[ism] + a * vx[ism] + b * vy[ism]; 
  float sig0=nn*sig_pad;
  sig=sqrt(sig0*sig0+sigthc*sigthc+sig_sfa*sig_sfa);
  return;
}

//-------------------------------------------------------------
float  RITERGeometry::Correct_thc(int id_sfm,int id_plm,float th_cer,float fi_cer) {
  float x0= dx_sph[id_sfm]+ dx_plm[id_plm];
  float y0= dy_sph[id_sfm]+ dy_plm[id_plm];
  th_cer=th_cer-(x0*cos(fi_cer)+y0*sin(fi_cer));
  return th_cer;	
}















