#include <riter/RITERMain.hh>
#ifdef RITER_DEBUG
#include "arte/Chrono.hh"
#endif

#define NBIN 256

#define NDQBIN 100
#define DQBIN  0.1666666

float DQ_RICH_rec_thc_xarray0[NDQBIN];
float DQ_RICH_rec_thc_xarray1[NDQBIN];
float DQ_RICH_rec_thc_xarray2[NDQBIN];
float DQ_RICH_rec_thc_xarray3[NDQBIN];


RITERIter::RITERIter(int Nhip, RITERAngle *theta_c,
		       vector<RITERTracks> *RTracks,
		       vector<RITERHits>   *RHits ){

  bin   =.45e-3;                  //! bin size
  nhip  = Nhip;
  Tracks =  RTracks;
  Fotons =  RHits;

  vector<RITERTracks> &tracks =*Tracks;
  vector<RITERHits>  &fotons =*Fotons;

  nhits = fotons.size();
  ntra  = tracks.size();
  niter = 0;
  float th_cer=0,fi_cer=0;
  int   itra  =0;
  int   calcok=0;
  int   is_inside_flag=0;

  rich.reserve(tracks.size()*fotons.size()/8);
#ifdef RITER_DEBUG
  cout << "Allocated for RICH table " << rich.capacity() << endl; 
  Chrono *timer=new Chrono("angle"); 

#endif

  for(vector<RITERTracks>::iterator it=tracks.begin();
      it!=tracks.end();++it){    
    int ifot=0;
    for(vector<RITERHits>::iterator hit=fotons.begin();
	hit!=fotons.end();++hit){

      RDEBUG( timer->start() ; )
      calcok=theta_c->Calculate((*hit),(*it),th_cer,fi_cer,is_inside_flag);
      RDEBUG( timer->stop(); )

      if (calcok==RITER_OK) {
        register int ir= (int) (th_cer/bin);
	if (ir > (NBIN-1) ) continue;
	register int icha=0;
	register float  utez=(*hit).utez;
	register float  w=(*hit).w;
	if (utez>0) {
	  rich.push_back(
	  RITERTable(ir,itra,ifot,icha,utez,th_cer,fi_cer,w,is_inside_flag));
	}
      }
      ifot++;
    }         //hits
    itra++;
  }           //tracks
  tablesize=rich.size();
#ifdef RITER_DEBUG
  RITERTime[13]=tablesize;
  cout << "Itertable.size()" << tablesize << endl;
  cout << " RITERAngle : Average time :" << timer->averageTime() << endl;
  RITERTime[4]= timer->averageTime();
#endif 
}

int RITERIter::Iterate(float &wdiff){
// iterational procedure

  if (!(tablesize>0)) return RITER_OK;
  float spf[nhits];	  // sum of probabilities over tracks for i-th photon
  float pf[NBIN][ntra];   // histogram (radius,track)
  // filling of histograms pf
  vzero(&pf[0][0],NBIN*ntra);
  for(vector<RITERTable>::iterator hit=rich.begin();
      hit!=rich.end();++hit){
    pf[(*hit).ir][(*hit).itra]+=(*hit).w;
  }
  // filanje spf
  vzero(spf,nhits);
  
  for(vector<RITERTable>::iterator hit=rich.begin();
      hit!=rich.end();++hit){
    spf[(*hit).ifot]+=pf[(*hit).ir][(*hit).itra];
  }
  //  cout << "spf filled\n";
  // nove utezi
  wdiff=0;
  for(vector<RITERTable>::iterator hit=rich.begin();hit!=rich.end();++hit){
    int ifot   =(*hit).ifot;
    int ir     =(*hit).ir;
    int itra   =(*hit).itra;
    float utez =(*hit).utez;
    float w_old=(*hit).w;
    float w=0;
    if(spf[ifot]>0) w=pf[ir][itra]/spf[ifot]*utez;
    wdiff=max(wdiff,fabs(w-w_old));
    (*hit).w = w;
#ifdef RITER_DEBUG_1
    myHistoFill(niter+2 ,0,0, 0, (*hit).thc, 0, w, tracks[itra].p);
#endif

  }
  niter++;
  return RITER_OK;
}

int RITERIter::Likelihoods(){
  vector<RITERTracks> &tracks = *Tracks;
  vector<RITERHits>   &fotons = *Fotons;

#ifdef  RITER_DEBUG
  Chrono *timer=new Chrono("angle"); 
  timer->start();
#endif

  // fill number of photons first
  for(vector<RITERTable>::iterator hit=rich.begin();
      hit!=rich.end();++hit){
    int   flag =  (*hit).flag;
    if (!flag) continue;
    int   itra = (*hit).itra;
    float wi   = (*hit).w;

    int type = ( flag > 0) ? 2 : 3; 
    tracks[itra].Likelihood()->Fill(type,wi);

  }  

#ifdef  RITER_DEBUG
  timer->stop();
  RITERTime[5]=timer->averageTime();
  timer->reset();
  timer->start();
#endif


  /// Calculate background photons
  for (vector<RITERTracks>::iterator itra=tracks.begin();
       itra!=tracks.end();++itra){
    (*itra).Likelihood()->SetBackgroundPhotons();
  }
  
#ifdef  RITER_DEBUG
  timer->stop();
  RITERTime[6]=timer->averageTime();
  timer->reset();
  timer->start();
#endif

  ///! fill log likelihoods
  for(vector<RITERTable>::iterator hit=rich.begin();
      hit!=rich.end();++hit){
    int   flag =  ( (*hit).flag > 0 );
    if (!flag) continue;

    register int   itra = (*hit).itra;
    register int   ifot = (*hit).ifot;
    register float thc  = (*hit).thc;
    register float wi   = (*hit).w;
    register int pmtype = fotons[ifot].Type(); 
    register float p    = tracks[itra].p;
    tracks[itra].Likelihood()->FillLKH(pmtype,thc,wi); 
    if ( RITERhisto ) {   
      if (p>30) {
	register int bin=(int) ( thc*DQBIN );
	switch ( pmtype ){
	case 0:
	  DQ_RICH_rec_thc_xarray2[bin]+=wi;
	  DQ_RICH_rec_thc_xarray0[bin]+=1 ;
	  break;
	case 1:
	  DQ_RICH_rec_thc_xarray3[bin]+=wi;
	  DQ_RICH_rec_thc_xarray1[bin]+=1 ;
	}
      }
    }

  }
#ifdef  RITER_DEBUG
  timer->stop();
  RITERTime[7]=timer->averageTime();
  timer->reset();
  timer->start();
#endif


  ///! calculate extended  likelihood function   
  for (vector<RITERTracks>::iterator itra=tracks.begin();
       itra!=tracks.end();++itra){
    (*itra).CalculateLKH();
  }

#ifdef  RITER_DEBUG
  timer->stop();
  RITERTime[8]=timer->averageTime();
#endif
  

  return RITER_OK;
}



int RITERIter::HFill(){

  vector<RITERTracks> &tracks = *Tracks;
  //  vector<RITERHits>   &fotons = *Fotons;
  for(vector<RITERTable>::iterator hit=rich.begin();
      hit!=rich.end();++hit){
    int   flag = (*hit).flag;
    if (!flag) continue;
    
    int   itra = (*hit).itra;
    //int   ifot = (*hit).ifot;
    float thc  = (*hit).thc;
    float wi   = (*hit).w;
    float fic  = (*hit).fic;    
    float p    = tracks[itra].p;
    int  itype = tracks[itra].itype;

    int isfm=0;
    int iplm=0;

    int type = ( flag > 0) ? 2 : 3; 

#ifdef RITER_DEBUG
    ntest(type,p,thc,wi);
    myHistoFill(0 , itype,isfm,iplm,thc,fic,1.,p );
    myHistoFill(1 , itype,0, 0, thc, 0, wi, p);
    //   printf("Hi %d %f %f %d %f\n",itra,thc,wi,itype,p); 
    htracks(itra,thc,wi,itype,p,flag);    
#endif
  }
  
  
  
  int ntra=0;
  for (vector<RITERTracks>::iterator itra=tracks.begin();
       itra!=tracks.end();++itra){
    (*itra).Test(ntra++);
  }
  return RITER_OK;
}









