#include <riter/RITERLikelihood.hh>

//-------------------------------------------------------------
RITERLikelihood::RITERLikelihood( int n, float *expected_thc,float *expected_photons, float sigm16, float sigm4){
  // constructor
  // input:
  // n number of hypothesis
  // thc cherenkov angle
  // total sigma m16,m4
  for (int i=0;i<4;i++) {
    count[i]=0;
    if (i==1) sig[i]=sigm4; else sig[i]=sigm16;
  }
  thc=expected_thc;  
  stf=expected_photons;
  SetWindow();

  for (int type=0;type < 4;type++) {  
    int nn=0;    
    for (int i=0;i<n;i++) {
      if (thc[i]==0) continue;
      min[type][nn  ]   = max(thc[i]-WindowWidth(type)*sig[type],0);
      max[type][nn  ]   = thc[i]+WindowWidth(type)*sig[type];
      nn++;
    }
    nval[type]=Windows(nn, &min[type][0],&max[type][0]); 
    SqrWindows(type);    
  }



  for (int i=0;i<NHIP;i++) lkh[i]=0;
  
  if(      ( A = Integral(2) )  ) {
    A = 1 / A;
  } else { 
    A = 0;
  }
  sfot=0;
  //Print();
}

//-------------------------------------------------------------
int RITERLikelihood::SqrWindows(int type){
 nval[type+4]= nval[type];
 for (int i=0 ; i<nval[type] ; i++ ){
   max[type+4][i]=sqr(max[type][i]);
   min[type+4][i]=sqr(min[type][i]);
 }
 return RITER_OK;
}
//-------------------------------------------------------------

int RITERLikelihood::Print(){
  for (int type=0;type < 4;type++) {  

    printf("window type =%d sigma=%f\n", type,sig[type]);

    for ( int i=0; i<nval[type];i++){
      printf("i =%d %f %f\n", i,min[type][i],max[type][i]);
    } 
    printf("-----------------------\n");
    printf("Integral %f\n",Integral(type));
    printf("-----------------------\n");
  }

  return 0;
};

int  RITERLikelihood::Fill(int type,float thc,float  weight){
  // Count the photons
  // window type 0-m16 1-m4 2-small 3-big
  // cherenkov angle of the photon track pair
  // weight of the photon
  if (weight==0) return RITER_ERROR;
  if ( IsInside(type, thc ) ) {
    //   cout <<  type << thc << " : " << count[type] << endl;
    count[type]+=weight;
    return RITER_OK;
  }
  return RITER_ERROR;
}
//-------------------------------------------------------------
float  RITERLikelihood::Fill(int type,float  weight){
  // Count the photons
  // window type 0-m16 1-m4 2-small 3-big
  // weight of the photon
  return count[type]+=weight;
}

//-------------------------------------------------------------
int  RITERLikelihood::FillLKH(int type,float thc1,float  weight){
  if ( thc1   == 0 )           return RITER_ERROR;   
  if ( weight == 0 )           return RITER_ERROR;

  float  stfb = BackgroundPhotons();
  static float sqrt2pi = sqrt(2*M_PI);

  sfot +=  weight;  
  if (stfb==0) {
    float konst = log( sqrt2pi * sig[type] );
    for (int itype=0;itype< NHIP;itype++) {
      if ( stf[itype] > 0 ) {
	float xx = (thc1-thc[itype])/sig[type];
        float f  = - (xx*xx/2+ konst );
	lkh[itype]+=weight*f;
      }
    }
    return RITER_OK;
  }

  float b   =  Background(thc1);
  float fb  =  log(b);
  for (int itype=0;itype< NHIP;itype++) {
    float f;
    if ( stf[itype] > 0 ) {
      float xx = (thc1-thc[itype])/sig[type];
      if (fabs(xx)<3) {
	float g = exp(-(xx*xx)/2)  / sqrt2pi /sig[type]; 
        f = log((stf[itype] * g + stfb * b )/( stf[itype]+ stfb));
  
      } else {
	f = log(stfb * b /( stf[itype]+ stfb) );
      }
      
    } else {
      f = fb;
    }
    lkh[itype]+=weight*f;
  }   
  return RITER_OK;
}
//-------------------------------------------------------------
int  RITERLikelihood::FillLKHtest(int type,float thc1,float *lkh){
  if ( thc1   == 0 )           return RITER_ERROR;   
  float weight=1;

  float stfb = BackgroundPhotons();
  static float sqrt2pi = sqrt(2*M_PI);

  sfot +=  weight;  
  if (stfb==0) {
    float konst = log( sqrt2pi * sig[type] );
    for (int itype=0;itype< NHIP;itype++) {
      if ( stf[itype] > 0 ) {
	float xx = (thc1-thc[itype])/sig[type];
        float f  = - (xx*xx/2+ konst );
	lkh[itype]+=weight*f;
      }
    }
    return RITER_OK;
  }

  float b   =  Background(thc1);
  float fb  =  log(b);
  for (int itype=0;itype< NHIP;itype++) {
    float f;
    if ( stf[itype] > 0 ) {
      float xx = (thc1-thc[itype])/sig[type];
      if (fabs(xx)<3) {
	float g = exp(-(xx*xx)/2)  / sqrt2pi /sig[type]; 
        f = log((stf[itype] * g + stfb * b )/( stf[itype]+ stfb));
  
      } else {
	f = log(stfb * b /( stf[itype]+ stfb) );
      }
      
    } else {
      f = fb;
    }
    lkh[itype]+=weight*f;
  }   
  return RITER_OK;
}
//-------------------------------------------------------------
int  RITERLikelihood::Windows(int n, float *x,float *y) {
  // concat connected intervals
  // the intervals should be ordered in a way that upper edges are descending
  if (n==0) return 0;
  int j=0;
  for (int i=1;i<n;i++) {
    if (x[j]<=y[i]) {
      x[j]=x[i];
    } 
    else {
      j++;
      y[j]=y[i];
      x[j]=x[i];
    }
  }
  j++;
  n=j;
  return n; 
}
//-------------------------------------------------------------
int RITERLikelihood::Subtract(float *a){
  float b[]={a[0],a[1],a[2],a[3]};
  if (b[3]>b[1]) {
     if (b[2]>b[0]) {
       if (b[2]<b[1]) a[1]=b[2];
       return 1;
     } else {
       return 0;
     }
  }else {
     if (b[2]>b[0]) {
       a[1]=b[2];
       a[2]=b[3];
       a[3]=b[1];
       return 2;
     } else {
       if (b[3]>b[0]) a[0]=b[3];
       return 1;
     }  
  }
}

//-------------------------------------------------------------
int  RITERLikelihood::AntiWindows(int &n, float *x,float *y,
				  int n1, float *x1,float *y1) {
  // combine two sets of intervals  C= (A & !B)
  // anti windowing
  // input :
  //   n,x,y   A set
  //   n1,x1,y1 B set
  // output:
  //  n,x,y   C set
  // return value n
  
  float min[2*NHIP];
  float max[2*NHIP];
  
  int j=0;
  for (int i=0;i<n;i++) {
    // printf("nov interval %d x=%f  y=%f\n",i,x[i],y[i]);
    min[j]=x[i];
    max[j]=y[i];
    int flag=1;
    for (int k=0; (x1[k] < y[i])&&(k<n1)&&(j<2*NHIP)&&(flag);k++) {
      float x[] = {min[j],max[j],x1[k],y1[k]};
      int   ix  = Subtract(x);
      switch (ix) {
      case 2:   
	min[j]=x[0];
	max[j]=x[1];
	j++;
	min[j]=x[2];
	max[j]=x[3];
	break;
      case 1:
	min[j]=x[0];
	max[j]=x[1];
	break;
      case 0:
        flag=0;
      }

    }
    j++;
  } 
  n=j;
  for (int i=0;i<n;i++) {
    x[i]=min[i];
    y[i]=max[i]; 
    //    printf(">>-%d x=%f  y=%f\n",i,x[i],y[i]);
  } 
  return j; 
}

//-------------------------------------------------------------
int RITERLikelihood::SetWindow(int normalm16, int normalm4,
			       int small    , int big      ){
  // example 3 sigma, 3 sigma , 3 sigma, 9 sigma 
  window[0]=normalm16;
  window[1]=normalm4;
  window[2]=small;
  window[3]=big;
  return 0;
}
//-------------------------------------------------------------
float RITERLikelihood::Integral(int type){
  float sum=0;
  for (int i=0;i<nval[type];i++)
    sum+= pow(max[type][i],4)- pow(min[type][i],4);
  return (sum/4);
}
//-------------------------------------------------------------
float RITERLikelihood::SetBackgroundPhotons(){
  Nanti  = Number(3);
  float s_narrow= Integral(2);
  float s_wide  = Integral(3);
  bgr =  Nanti * s_narrow/(s_wide-s_narrow);
  return bgr;
};

//-------------------------------------------------------------














