#include <riter/RITERMain.hh>
float RITERTime[20];

RITERMain::RITERMain(){

  if (!RITER_Geometry) RITER_Geometry= new RITERGeometry();
  RITER_geo= RITER_Geometry;
  Faktor  =new RITERFaktor();
  if ((RITERnhip<NHIP)&&(RITERnhip>1))   nhip=RITERnhip;
  else nhip=NHIP;
  ThetaObj =new RITERAngle(nhip);
  mode=RITERinput;
  cout << " RITER: number of likelihood hypothesis" << nhip << endl;
  niter=20;
  cout << " RITER: number of iterations" << niter << endl;

};
//-------------------------------------------------------------

int RITERMain::main()
{ 
  float wdiff=0;
#ifdef RITER_DEBUG
  cout << " RITERMain::main(): Hits: " << fotons.size() << " Tracks:" << tracks.size() <<endl;

#endif
  if (tracks.size()==0) return RITER_OK;
  if (fotons.size()==0) return RITER_OK;
  if (tracks.size()> 1000) return RITER_ERROR;
  if (fotons.size()> 15000) return RITER_ERROR;
#ifdef  RITER_DEBUG
  Chrono *timer=new Chrono("angle"); 
  timer->start();
#endif

  RITERIter *iterational_table= new RITERIter( nhip,ThetaObj,&tracks, &fotons);

#ifdef  RITER_DEBUG
  timer->stop();
  RITERTime[0]=timer->averageTime();
  timer->reset();
  timer->start();
#endif

  for (int iter=0;iter<niter;iter++) {
    iterational_table->Iterate(wdiff);
  }

#ifdef  RITER_DEBUG
  timer->stop();
  RITERTime[1]=timer->averageTime();
  timer->reset();
  timer->start();
#endif

  iterational_table->Likelihoods();

#ifdef  RITER_DEBUG
  timer->stop();
  RITERTime[2]=timer->averageTime();
  timer->reset();
  timer->start();
#endif
 
#ifdef RITER_DEBUG  
  myevnt(fotons.size(),tracks.size());
  for(vector<RITERTracks>::iterator it=tracks.begin();
      it!=tracks.end();++it){
    FillNtuple( (*it) );
  }
  //  iterational_table->HFill();
#endif  

#ifdef  RITER_DEBUG
  timer->stop();
  RITERTime[3]=timer->averageTime();
#endif
  return RITER_OK;
}; 

//-------------------------------------------------------------
// transformation from ARTE unit point and direction s to riter RICH entrance  
RITERVec   RITERMain::toRICH(float x,float y,float z,RITERVec s){
return 10.*RITERVec(x,y,z)+((RITER_geo->z_rich_min-10*z)/s.z())* s;
};
  
RITERVec    RITERMain::toRICH( RITERVec &r, RITERVec s){
return 10.*r+((RITER_geo->z_rich_min-10*r.z())/s.z())* s;
};
//-------------------------------------------------------------      
int RITERMain::itip_lund(int ipart) {
  
  // return =1  electron
  //        =2  muon
  //        =3  pion
  //        =4  kaon
  //        =5  proton
  //        =0   ?

  if (abs(ipart)== 11)  return 1; 
  if (abs(ipart)== 13)  return 2; 
  if (abs(ipart)== 211) return 3; 
  if (abs(ipart)== 321) return 4; 
  if (abs(ipart)== 2212) return 5; 
  if  (abs(ipart) < 6) return 0;
  return 0;
};
//------------------------------------------------------------- 
int RITERMain::AddPhoton(int sm ,int r,int c,int pos, int ch,int type) {

  int ir = r*2+pos/2;
  int ic = c*2+pos%2;
  int ism = sm; 
  if (ism >6) {
    ism=20-ism;
    ic=11-ic;
    ir=31-ir;
  }
  if (RICH_Detector->ChannelExist(ism,ir,ic,ch)) {
    RITERVec rf=toRITERVec(RICH_Detector->pos()) ;
    int dn=rf.y()<0;
    rf=virtual_position(rf, RITER_geo->Plr0(dn), RITER_geo->Plvn(dn));
    float sig  = sqrt(sqr(  (type+1)*
			    RITER_geo->sig_pad )+ 
		      sqr( RITER_geo->sigthc  )+
		      sqr( RITER_geo->sig_sfa ));
    fotons.push_back(RITERHits(rf,sig,1));
    //    printf("AddHit  sm %d row %d col %d ch %d \n",ism,ir,ic,ch);
  } else {
    //    printf("AddHit ERROR sm %d row %d col %d ch %d \n",ism,ir,ic,ch);
    return RITER_ERROR;
  }
  //      src2det(ism,ir,ic,ch,type,msm,x1,y1);   
  return RITER_OK;
};
//------------------------------------------------------------- 

int  RITERMain::AddTrack( RITERVec &dir,float err,RITERVec &point, float p)
{ 
  //  point should be in [cm]

  //register float zmin= RITER_geo->z_rich_min;
  if((point.z()>300)&& 
  (fabs(dir.x())< RITER_geo->th_0) &&  
  (fabs(dir.y())< RITER_geo->fi_0)  ) {
    RITERVec r =toRICH(point,dir);
    RITERTracks Track(r,dir,p,sqrt(err),0);
    //Track.Print(); 
    tracks.push_back(Track);
    return 1; 
  }
  return 0;
};
// ------------------------------------------------
int RITERMain::AddTrack(float x, float y, float z, float tx, float ty, float p, 
	     float sx, float sy, float sz, float stx, float sty, float sp)
{
  //  x,y,z  should be in [mm]

  RITERVec r(x/10,y/10,z/10);
  //  cout << r << endl;
  RITERVec dir= RITERVec(tx,ty,1).unit();
  float err=stx*stx+sty*sty;
  AddTrack(dir,err,r,p);
  return RITER_OK;
};




