#include <riter/RITERPlanarmirror.hh>

RITERPlanarmirror::RITERPlanarmirror(){
  if (!RICH_Vessel)   printf("RICH_Vessel not initialized\n");
  RICHVessel   &rv=*RICH_Vessel;

  // reflection  plane
  rp_mu=  toRITERVec(rv.plane_mirror->SingleMirror(3,0)->x0()); 
  rp_md=  toRITERVec(rv.plane_mirror->SingleMirror(0,0)->x0()); 
  ezp_mu= toRITERVec(rv.plane_mirror->SingleMirror(3,0)->ez()); 
  ezp_md= toRITERVec(rv.plane_mirror->SingleMirror(0,0)->ez()); 

  pl_r0[0] = rp_mu;
  pl_r0[1] = rp_md; 
  pl_vn[0] = ezp_mu;
  pl_vn[1] = ezp_md; 

  float coldivision=753;   // how to read this from ARTE ????
  float rowdivision=638;   // how to read this from ARTE ????
  float xsiz=coldivision*rv.plane_mirror->numcol();
  float ysiz=rowdivision*rv.plane_mirror->numrow()/2.0;
 
  pl_vx[0] = xsiz*toRITERVec(rv.plane_mirror->SingleMirror(3,0)->ex());
  pl_vx[1] = xsiz*toRITERVec(rv.plane_mirror->SingleMirror(0,0)->ex());  
  pl_vy[0] = ysiz*toRITERVec(rv.plane_mirror->SingleMirror(3,0)->ey());
  pl_vy[1] = ysiz*toRITERVec(rv.plane_mirror->SingleMirror(0,0)->ey());
  
  //cout << "plvx" << pl_vx[0] << pl_vx[1];
  //cout << "plvy" << pl_vy[0] << pl_vy[1];

  pl_dx[0] = pl_vx[0].mag2();  
  pl_dx[1] = pl_vx[1].mag2();
  pl_dy[0] = pl_vy[0].mag2();  
  pl_dy[1] = pl_vy[1].mag2();

  // corrections

  for (int i=0;i<35;i++) { 
         dx_plm[i]=0;
         dy_plm[i]=0;
  } 

}

int  RITERPlanarmirror::Which_PLM( RITERVec  &r0, RITERVec  &dir,int which,float &path ){
  // finds the ID of the single plane mirror, where the line R0,DIR crosses it
  //  routine not optimised

  //    RITERVec r, dir         ! line (dir not needed to be unit vector)
  //    int which                 ! 0=upper , 1=lower plane mirror
  //    float path                ! length along line from R0 to mirror
  //    return value              ! ID of single mirror (0:35 or -1 if outside)

  RITERVec r=r0- pl_r0[which];
  float s1=r* pl_vn[which];
  float s2=dir* pl_vn[which];
  if (!s2) return -1;
  path = -s1/s2;
  r=r+path*dir;
  float x=r* pl_vx[which];
  x=x/ pl_dx[which];
  float y=r* pl_vy[which];
  y=y/ pl_dy[which];
  int id= (int)(6*x)+6*(int)(3*y)+(1-which)*18;
  if ((id>34)||(id<0)){
    return -2;
  } 
  
  return id;
}

int RITERPlanarmirror::PrintPlanarmirror(){

  cout << "RITERPlanarmirror " << endl;
  cout << "pl_r0 pl_vx pl_vy pl_vz" << endl;
  for (int i=0;i<2;i++){ 
    cout << i <<  ":" << pl_r0[i] << pl_vx[i] << pl_vy[i] << pl_vn[i] <<  endl;

  }
  return RITER_OK;


}
