#include <riter/RITERSupermodule.hh>

RITERSupermodule::RITERSupermodule(){
  if (!RICH_Detector) printf("RICH_Detector not initialized\n");
  if (!RICH_Vessel)   printf("RICH_Vessel not initialized\n");
  RICHDetector &rd=*RICH_Detector;
  RICHVessel   &rv=*RICH_Vessel;
  float d_pad=9;  

  RITERVec rp_mu=  toRITERVec(rv.plane_mirror->SingleMirror(3,0)->x0()); 
  RITERVec rp_md=  toRITERVec(rv.plane_mirror->SingleMirror(0,0)->x0()); 
  RITERVec ezp_mu= toRITERVec(rv.plane_mirror->SingleMirror(3,0)->ez()); 
  RITERVec ezp_md= toRITERVec(rv.plane_mirror->SingleMirror(0,0)->ez());

  RITERVec rp_m,ezp_m;
  float xlen=432;           // dimensions of sm [mm]
  float ylen=1152;          // dimensions of sm [mm]

  for (int i=0;i<14;i++){  
    vx[i]= xlen*toRITERVec(rd.SuperModule(i).ex()); // 
    vy[i]= ylen*toRITERVec(rd.SuperModule(i).ey());
    vn[i]= -toRITERVec(rd.SuperModule(i).ez());   
    r0[i]= toRITERVec(rd.SuperModule(i).x0());

    //if (i> 6)      r0[i]+=vx[i]+vy[i];


    if (i>6) {  
      rp_m=rp_md;
      ezp_m=ezp_md;
    } else {
      rp_m=rp_mu;
      ezp_m=ezp_mu;
    }
    

    r0[i]=virtual_position(r0[i],rp_m,ezp_m);
    vx[i]=virtual_direction(vx[i],ezp_m);
    vy[i]=virtual_direction(vy[i],ezp_m);
    vn[i]=virtual_direction(vn[i],ezp_m);
  }
  
  for (int k=0;k<14;k++) {
    dvx[k]=vx[k].mag2();
    dvy[k]=vy[k].mag2();
  }
  for (int k=0;k<6;k++) {
    float d=(r0[k]+vx[k]-r0[k+1]).mag2();
    dxsm[k]=(sqrt(dvx[k])+sqrt(d))/d_pad ; // units: [padsize]
    //fprintf(stdout,"%2d %f\n", k ,dxsm[k] );
  }
  
  for (int k=7;k<13;k++) {
    //vx[k]=-vx[k];
    //vy[k]=-vy[k];
    float d=(r0[k]+vx[k]-r0[k+1]).mag2();
    dxsm[k]=(sqrt(dvx[k])+sqrt(d))/d_pad ; // units: [padsize]
    //    fprintf(stdout,"%2d %f\n", k ,dxsm[k] );
  }


  for (int ism=0;ism<NUMSM;ism++){
    for (int ir=0;ir<NUMROW;ir++){
      for (int ic=0;ic<NUMCOL;ic++){
	for (int ich=0;ich<NUMCH;ich++){
	  
	  if(!rd.ChannelExist(ism,ir,ic,ich)) {
	    Set(RITERVec(),-1,ism,ir,ic,ich);
	    continue;
	  }
          RITERVec r;
	  int type=4/rd.pm().numcol()-1;

	  if (ism<7) {
	    r=virtual_position(toRITERVec(rd.pos()),rp_mu,ezp_mu);
	    Set(r,type,ism,ir,ic,ich);
	  } else { 
	    r=virtual_position(toRITERVec(rd.pos()),rp_md,ezp_md);
	    Set(r,type,ism,ir,ic,ich);
	  }
//            float  alpha=0,beta=0;
//            int sm=0;
//            DetectorIntersection(r,alpha,beta,sm);
//  	  cout << sm <<"Is detector" << r - Position(sm,beta,alpha) << RITERVec(ism,ir,ic) << RITERVec(sm,int(beta/4),int(alpha/4))<< endl;
//            RITERVec r2;
//            DetectorIntersection(r,RITERVec(0,0,-1),(ism>6)*7,r2,alpha,beta,sm);
//            cout  << sm <<"DetectorIntersection" << RITERVec(sm,int(beta/4),int(alpha/4)) << Type(sm,beta,alpha) <<" END " <<endl;
          
	}
      }
    }
  }
  
}

int  RITERSupermodule::IsDetector(RITERVec &r,float  &alfa,float  &beta,int ism ){
 
  RITERVec p0= r - r0[ism];
  alfa=(p0* vx[ism])/ dvx[ism];
  beta=(p0* vy[ism])/ dvy[ism];
  return RITER_OK;
} 

int  RITERSupermodule::DetectorIntersection(RITERVec &r,
					    float  &alfa,float  &beta,int &ism)
  // not optimized
  // int kje;               // 0=up, 7=down
  /// r .... virtual coordinate on the detector
{
  RITERVec p0;
  int kje=(r.y()<0)*7;
  alfa=-1;
  for (int k=kje+6;((k>=kje)&&(alfa<0));k--) {
    ism=k;
    IsDetector(r,alfa,beta,ism);
  }
  alfa*=48.;              // m16 pad units
  beta*=128.;             // m16 pad units
  return RITER_OK;
};

//-------------------------------------------------------------
int  RITERSupermodule::DetectorIntersection(RITERVec &r,RITERVec &dir,int kje,RITERVec &r1,float  &alfa,float  &beta,int &ism)
  // not optimized
  //int kje;               // 0=up, 7=down
{
  RITERVec p0, p;
  float sn=0;
  alfa=-1;
  for (int k=kje+6;((k>=kje)&&(alfa<0));k--) {
    ism=k;
    sn=dir* vn[k];
    if(sn==0) continue;
    p0=r- r0[k];
    float s=p0* vn[k];
    s=-s/sn;
    p=p0+s*dir;
    alfa=(p* vx[k])/ dvx[k];
    //if (k>6) cout << "DETIN" << k << " :" << alfa << endl;
  }
  if(sn==0) {
    alfa=1000.;
    beta=1000.;
    p=alfa* vx[ism]+beta* vy[ism];         
  } else {
    beta=(p* vy[ism])/ dvy[ism];
  }
  r1=p+ r0[ism];
  alfa*=48.;              // m16 pad units
  beta*=128.;             // m16 pad units
  return RITER_OK;
};
//-------------------------------------------------------------
int  RITERSupermodule::PrintSupermodule(){
  cout << "RITERSupermodule " << endl;
  cout << "r0 , vx , vy , vn" << endl;
  for (int i=0;i<14;i++){ 
    cout << i <<  ":" << r0[i] << vx[i] << vy[i] << vn[i] <<  endl;
    cout << i << "##" << dvx[i]<<"##" << dvy[i] <<"##" <<  dxsm[i] << endl; 

  }
  return RITER_OK;
}













