#include <riter/RITERTracks.hh>

RITERTracks::RITERTracks(
	      const RITERVec &rtrack, 
              const RITERVec &dirtrack,  
	      float momentum, 
	      float sigma,  
              int   type=0, ArtePointer<RTRA> artetrack ){
#ifdef RITER_DEBUG
  //   cout << "RITERTracks()\n"; 
#endif
   if (!RITER_Geometry) RITER_Geometry=new  RITERGeometry();

    geo=RITER_Geometry;
    r  =rtrack;
    dir=dirtrack;
    //~~~~~~~~~~~~~~~ PAZIIIIIIIII
    //dir=RITERVec(dir.y(),dir.x(),dir.z());
    //~~~~~~~~~~~~~

    sig   = sigma;
    p     = momentum;
    rtra  = artetrack;
    itype = type;
    sfot  = 0; 

    for (int i=0; i<NHIP; i++) {
      plkh[i] =0;
      stf[i]  =0;
      acg[i]  =0;
    }
    likelihood = NULL;
    Calculate();

   };


int  RITERTracks::Calculate()

// track should already be on rich entrance 
{ 
  int nhip=NHIP;
  if ((RITERnhip<NHIP)&&(RITERnhip>1))   nhip=RITERnhip;   
  RITERVec  r1, r2;
  float path;
  // particle hit on upper and lower mirror spheres
  for (int mm=0; mm<2;mm++ ) {
    if (!SphereIntersection(r,dir,geo->Radius(mm),geo->CoC(mm),tr1[mm],path)){
      cout << r << dir << tr1[mm] << endl;
      printf("RITERTracks::Calculate: presek_sfera=.false.\n");
      printf("RITERTracks::Calculate: This will be wrong\n");
    }
  }

  // which mirror is actually hit
  // geo->CoC(mm)   mm 0 :upper, mm 1 lower
 
  wm=(tr1[0].y()>=0);

  int kaj =(tr1[0].y()<0) ;
  // beam shroud

  if ( (sqr(r.x())+sqr(r.y())) < sqr(geo->GetBeamRadius()) ) {

    wm=2;
    if (TubeIntersection(r,dir,geo->GetBeamRadius(),r1,r2)) {
      r=r1+dir; // put it slightly (1mm) outside-num.error
      if (r.z()> tr1[kaj].z()) {
        wm=3;
        tpath=0;
	return RITER_ERROR;
        
      }
    } else {
      // no intersection with the tube (track paralell to the tube)
      wm=4;
      tpath=0;
      return RITER_ERROR;       
    }
  }
  // mean photon emission point
  r0=(r+tr1[kaj])*0.5;   
  tpath=2*(r0-r).mag();
  float a[2],b[2];
  int ism[2];
  for (int mm=0;mm<2;mm++) {
    tdir1[mm]=dir;
    r1=tr1[mm];
    SphereReflection(r1,tdir1[mm],geo->Radius(mm),geo->CoC(mm));
    geo->DetectorIntersection(tr1[mm],tdir1[mm],mm*7,trc[mm],a[mm],b[mm],ism[mm]);
    id_plm[mm]= geo->Which_PLM(tr1[mm],tdir1[mm],mm,path);
  }  
  id_sfm[kaj] = geo->Which_SFM(tr1[kaj]);
  // expected cerenkov angles, number of expected signal photons


  r1=trc[kaj];
  //float acg[NHIP];

  for (int i=1;i<NHIP;i++) {
    thc[i]=CherenkovAngle(p,i,geo->ref_ind);
    ExpectedPhotons(thc[i],tpath,stf[i],acg[i],acgm16[i]);
  }
  thc[0]    = 0;
  stf[0]    = 0;
  acgm16[0] = 1;

  return RITER_OK;
};
//-------------------------------------------------------------

int  RITERTracks::ExpectedPhotons(float thc,float path, 
				  float &stfs, float &acg,float &acg16)
  
  // calculates expected number of photons
  //    float thc;   // expected cerenkov angle
  //    float path;  // track length in the radiator
  //    float stfs, acg;
{     

  
  stfs=geo->an_zero*sqr(sin(thc))*path;
  float ptip[2]={0,0};
  acg=Acceptance(thc,ptip);
  stfs*=acg;
  acg16=ptip[0];
#ifdef RITER_DEBUG_1
    cout <<  thc << "  RITERTracks::ExpectedPhotons " << stfs << " Acc:" << acg ;
   cout <<  "  ["<<ptip[0] << "," << ptip[1] <<  "]   "<< path << "\n"; 
#endif

  stfs *= 0.7;
  return RITER_OK;
};

//-------------------------------------------------------------   
int      myprofFill(int id,float n1,float n2, float weight);
  

RITERLikelihood * RITERTracks::Likelihood(){ 
    if (!likelihood) {
      likelihood=new RITERLikelihood(NHIP,thc,stf,
				     sqrt(sqr(sig)+sqr(geo->Sigma(0))),
				     sqrt(sqr(sig)+sqr(geo->Sigma(1))) );
    }
    return likelihood;
  }

#ifdef RITER_DEBUG    
int FillTest(int iparti,float  p , float sfot, float bgr, float thc, float *lkh,float *stf);

int   htracks(int itra,int itype,float p, float ,float);
#endif

int RITERTracks::CalculateLKH(){   
  float bgr    =   Likelihood()->BackgroundPhotons();
  sfot         =   Likelihood()->SignalPhotons();
  for (int i=0;i<NHIP;i++) {
    plkh[i]    =   Likelihood()->Get(i) + pois_log( stf[i] + bgr , sfot );
  }  

  return RITER_OK; 
}

int RITERTracks::Test(int itra){   
#ifdef RITER_DEBUG    
  float bgr    =   Likelihood()->BackgroundPhotons();
  sfot         =   Likelihood()->SignalPhotons();

  
  static int ipart=0;
  for (int i=0;i<100;i++) {
    float lkh[NHIP]={0,0,0,0,0};
    float thc=i/100.*0.06;
    Likelihood()->FillLKHtest(0,thc, lkh);
    FillTest(ipart, p , sfot, bgr, thc, lkh,stf);
    htracks(itra,itype,p,thc,exp(lkh[itype])*(bgr+stf[itype])*0.0006);
  }
  ipart++;
  
  

  myprofFill(itype,stf[itype]+bgr, sfot,1); 
#endif
  return RITER_OK; 
}

//-------------------------------------------------------------      


float RITERTracks::Acceptance(float thc, float *ptip ) {
  // acceptance: portion of a detected ring
  // screening of a beam pipe, detector active area
  // using ray tracing
  // input thc : ring radius (radians)
  // output ptip: relative portion of M16 and M4
  // return value acceptance
  float path=0;
  ptip[0]=0;
  ptip[1]=0;
#define  ACGNN 100    
  static float dfi=2*M_PI/ ACGNN;
  static float cfi=cos(dfi);
  static float sfi=sin(dfi);
  float zn=0;
  int k_tr= (tr1[0].y()<0);
  int k0=0;
  RITERVec dirf,dirf1,r1,r2;
  RITERVec dirnn(sin(thc),0,cos(thc));
  for (int iray=0;iray<ACGNN;iray++){
    // rotation around z-axis
    dirnn= RITERVec( dirnn.x()*cfi+dirnn.y()*sfi,
		      -dirnn.x()*sfi+dirnn.y()*cfi,
		       dirnn.z());	
    dirf= RotateToFixed(dir,dirnn); 
    k0=k_tr;
    if(!SphereIntersection(r, dirf ,geo->Radius(k0),geo->CoC(k0),r1,path) ) 
      { continue; }
    int k= (r1.y()<0) ;
    if (k!=k0) {
      k0=k; 
      if (!SphereIntersection(r,dirf,geo->Radius(k0),geo->CoC(k0),r1,path)) 
	{ continue; }
    }

    path=0;
    int seka=ShroudIntersection(r,dirf,geo->GetBeamRadius(),geo->GetBeamDir(),r2,path);
    if ( (seka)  &&   (path>0)   &&   ( r2.z() < r1.z() ) ) {  continue; }
    
    // reflection from a non-segmented spherical mirror
    RITERVec u  = ( r1-geo->CoC(k0) )*(1/geo->Radius(k0));
    float      a  = u*dirf;
    dirf1         = dirf-2*a*u;

    seka    = ShroudIntersection(r1,dirf1,geo->GetBeamRadius(),geo->GetBeamDir(),r2,path);
    if ( (seka) &&   (path>0)   &&   ( r2.z()> geo->z_rich_min) ) { continue; }
    float col  = 0, row = 0;
    int   ispm = 0;
   
    geo->DetectorIntersection(r1,dirf1,7*k0,r2,col,row,ispm);
    int itip = geo->Type(ispm,row,col);

    if (k0!=k_tr) {
      float dd=r1.y()-tr1[k_tr].y();
      float pp= (dd==0) ? 0 : r1.y()/dd ;
      if ( itip>=0 ) { 
	zn+=pp;
        ptip[itip]+=pp;
      }
      
      // reflection from a non-segmented spherical mirror
      u     = ( tr1[k_tr]-geo->CoC(k_tr) ) *(1/geo->Radius(k_tr));
      a     = u*dirf;
      dirf1 = dirf-2*a*u;

      geo->DetectorIntersection(tr1[k_tr],dirf1,7*k_tr,r2,col,row,ispm);
      itip = geo->Type(ispm,row,col);
      if ( itip>=0 ) { 
	zn+=(1-pp);
	ptip[itip]+=(1-pp);
      }
      
    } else {
      if ( itip>=0 ) { 
	zn++;
	ptip[itip]++;
      }
    }

  }
 float s= ptip[0]+ptip[1];
 if (s>0) {
   ptip[0]/=s;
   ptip[1]/=s;
 }
 return zn/ACGNN;
}
//-------------------------------------------------------------      

int RITERTracks::Print() {
  cout << "**************tracks_par************************" << endl;
  cout << "point at the entrance of RICH " << r << endl;
  cout << "direction of the track        " << dir << endl;
  cout << "momentum                      " << p << endl;
  cout << "sigma (dir)                   " << sig << endl;
  cout << "itype                         " << itype << endl;
  cout << "mean photon emmision r0       " << r0 << endl;
  cout << "particle hit tr1[0]              " << tr1[0] << endl;
  cout << "particle hit tr1[1]              " << tr1[1] << endl;
  cout << "particle dir  tdir1[0]            " <<tdir1[0] << endl;
  cout << "particle dir  tdir1[1]            " << tdir1[1] << endl;
  cout << "ring centers  trc[0]             " << trc[0] << endl;
  cout << "ring centers  trc[1]             " << trc[1] << endl;
  cout << "which mirror   =1 upper       " << wm << endl;
  cout << "expected cerenkov angles for {null,e,mu,pi,k,p}  " ;
  for (int j=0;j<NHIP;j++) cout << thc[j] << " ";
  cout << endl;
  cout << "expected number of signal photons for {..}  " ;
  for (int j=0;j<NHIP;j++) cout << stf[j] << " ";
  cout << endl;
 return RITER_OK;
};


//-------------------------------------------------------------
float  RITERTracks::CherenkovAngle(float p,int itype,float ref_ind)
  
  // calculates cerenkov angle
{
  static  const float  p_mass[6]= {0,         // gamma
				   .511e-3,   // electron  
				   105.66e-3, // muon  
				   139.57e-3, // pion
				   493.68e-3, // kaon
				   938.27e-3  // proton 
 };
  
  float mass=p_mass[itype];
  if((mass<=0)||(p<0)) {
    cout << "::cher_angle  error: mass = " <<  mass << endl;
    cout << "::cher_angle  momentum    = " << p << endl ;
    cout << "::cher_angle  STRANGE !!! " << endl;
  }
  float a=p/mass;
  float b=a/sqrt(1.+a*a)*ref_ind;
  if(b>1)  return acos(1./b);
  else     return 0. ;
};
//------------------------------------------------------------- 















