#include <riter/RITERUtils.hh>

inline float com(RITERVec &dir,int izb)
{
  if (izb==1) return dir.x();  
  if (izb==2) return dir.y();  
  if (izb==3) return dir.z();
  return 0;
};

void premica(RITERVec &rinit,RITERVec &dir,RITERVec &rfinl,float  &path,int izb)

//to calculate the second point or the distance between two points
//of line given by:
//rinit   initial point (input)
//dir     unit vector of line direction (input)
//rfinl   final point (input/output)
//path    distance between initial and final points (input/output)
//depending on parameter
//izb =0  path given
//    =1  rfinl(1) given
//    =2  rfinl(2) given
//    =3  rfinl(3) given
//calculates three other unknowns
	
{
  if(izb) {
    
    if(com(dir,izb)) path=(com(rfinl,izb)-com(rinit,izb))/com(dir,izb);

    else {
      cout << "line perpendicular to axis:" << izb << endl;
      cout << " STRANGE !!!! " << endl; 
    }
  }
  rfinl=rinit+path*(dir);
  return;
};


RITERVec transform_from_fixed( RITERVec  &zdir, RITERVec  &vin ){
  // transformation of a vector from fixed system S to current system S'
  // such that x' is in plane (x,z) -  horizontal in HERA-B
  // ZDIR() is unit vector of z-axis of S' given in fixed system S
  // VIN()  are the coordinates of a vector in S
  // return  are the coordinates of a vector in S'
  float ss=1.-zdir.y()*zdir.y();                                                 
  if (ss<=0) return RITERVec(vin.x(),vin.y()*zdir.y(),-vin.z()*zdir.y());
  else {
    ss=sqrt(ss);
    RITERVec xdir(zdir.z()/ss,0,-zdir.x()/ss); 
    RITERVec ydir=zdir.cross(xdir);
    return RITERVec(xdir*vin,ydir*vin,zdir*vin);
  }
  return RITER_OK;
}

RITERVec RotateToFixed( RITERVec &smer_z, RITERVec &vin)
//transformation of a vector from current system s' to fixed system s 
//smer_z() is unit vector of z-axis of s' given in fixed system s 
//vin()  are the coordinates of a vector in s'
//vout() are the coordinates of a vector in s
{
      if(smer_z.z()==1)  return vin;
      else if(smer_z.z()==-1) return -vin;
      else {
         float cth=smer_z.z();
         float sth=sqrt(1.-cth*cth);
         float cfi=smer_z.x()/sth;
         float sfi=smer_z.y()/sth;
       //rotation by theta around y axis
         float x=cth*vin.x()+sth*vin.z();
         float y=vin.y();
         float z=-sth*vin.x()+cth*vin.z();
       //rotation by fi around z axis
     
         return RITERVec(cfi*x-sfi*y,sfi*x+cfi*y,z);
      }
   
}

int SphereIntersection( RITERVec  &r,RITERVec  &dir,float radij, RITERVec  rc, RITERVec  &r1, float &path)

//crossection of a line r(),dir() and sphere of radius radij and center rc()
//r1() is concave r2() convex crossection if seka=.true.
{
  RITERVec r0 =r-rc;
  float rdir=r0*dir;
  float rr  =r0.mag2();
  float d=rdir*rdir+radij*radij-rr;
  if(d<0) {
    return 0;
  } else {
    
    d=sqrt(d);
    path = d-rdir;
    r1=r + path *dir;
    return 1;    
  }

};    

int TubeIntersection( RITERVec  &r,RITERVec  &dir,float radij, RITERVec  &r1, RITERVec  &r2)
  // cross section of a line R(),DIR() with a tube of radius RADIJ and
  // axis along z at RC
  // R2() is convex crossection if SEKA=.true.
{ 
  float b=sqr(dir.x())+sqr(dir.y());
  if (b==0) return 0;

  float a=r.x()*dir.x()+r.y()*dir.y();

  float d=a*a-b*( r.x()*r.x() + r.y()*r.y() -radij*radij );
  if ( d < 0 )  return 0;
  
  d=sqrt(d);
  float path=(d-a)/b;
  r1=r+path*dir;
  path=-(d+a)/b;  
  r2=r+path*dir;  
  return 1;
};    


int ShroudIntersection( RITERVec  &r,RITERVec  &dir,float radij, RITERVec  rc, RITERVec  &r2, float &path)
  // cross section of a line R(),DIR() with a tube of radius RADIJ and
  // axis along z at RC
  // R2() is convex crossection if SEKA=.true.
{ 
  float a=sqr(dir.x())+sqr(dir.y());
  if (a==0) return 0;
  float x=r.x()-rc.x();
  float y=r.y()-rc.y();
  float b=dir.x()*x+dir.y()*y;
  float c=x*x+y*y-radij*radij;
  float d=b*b-a*c;
  if (d<0) return 0;
  d=sqrt(d);
  path=-(d+b)/a;
  r2=r+path*dir;  
  return 1;
};    

void SphereReflection(RITERVec &r,RITERVec &dir,float radij,RITERVec rc)

//geometrical reflection on a sphere of radius radij and with center at rc()
//r() on input:  point where a ray hits the sphere
//    on output: focus, i.e. a point where reflected ray crosses the optical
//		 axis (opt. axis is paralel to impact ray)
//dir() on input: ray's direction (unit vector) before reflection
//      on output: ray's direction (unit vector) after reflection

{
  RITERVec n = (r-rc).unit();
  float  a     = dir*n;      //scalar product
  r            = radij/2./a*dir+rc;
  dir          = dir-2.*a*n;
  return;
};
//-----------------------------------------------------------
#ifndef NO_ARTE
float   Momentum(ArtePointer<RSEG> rseg){
  float zm=450;
  float xm=rseg->xf+rseg->txf*(zm-rseg->zf);
  float alpha_x0=atan2(xm,zm);
  float phi=atan(rseg->txf)-alpha_x0;
  float par1=0.625;  // from fit
  float cosalpha_y=1/sqrt(sqr(rseg->tyf)+1);
  return fabs(par1/phi/cosalpha_y);
}
#endif
//-----------------------------------------------------------
float   Momentum(float xf,float zf,float txf,float tyf){
  float zm=450;
  float xm=xf+txf*(zm-zf);
  float alpha_x0=atan2(xm,zm);
  float phi=atan(txf)-alpha_x0;
  float par1=0.625;   // from fit
  float cosalpha_y=1/sqrt(sqr(tyf)+1);
  return fabs(par1/phi/cosalpha_y);
}

//-----------------------------------------------------------
#ifndef NO_ARTE
ArtePointer<RCCL> Find_ecal(ArtePointer<RSEG>rseg){
  
  RITERVec r_otr=RITERVec( rseg->xf, rseg->yf, rseg->zf);
  RITERVec dir_otr=RITERVec( rseg->txf, rseg->tyf, 1);
  float z_ecal=1350;
  r_otr+=dir_otr*(z_ecal-r_otr.z());
  ArtePointer<RCCL> matchrccl;
  float dxold=3;
  float dyold=20;
  for(ArteTable<RCCL>::iterator rcclit=ArteTable<RCCL>::begin();
      rcclit!=ArteTable<RCCL>::end();++rcclit) {
    ArtePointer<RCCL> rccl(rcclit);
    float x_offset=-0.3;
    float y_offset=2.2;
    float dx=fabs(rccl->x-r_otr.x()-x_offset);
    float dy=fabs(rccl->y-r_otr.y()-y_offset);
    if ( (dx < dxold) && (dy < dyold) ) {
      matchrccl=rccl;
      dxold=dx;
      dyold=dy;
    }
    
  }      
  return matchrccl;
}

#endif
//-----------------------------------------------------------




