!-----------------------------------------------------------------------
! cerenkov angle calculation for track-photon pair
! M. Staric, jan-2000
!
!-----------------------------------------------------------------------


      INTEGER FUNCTION CERENKOV_KOT(R0,DIR,RH,RC,R_MI,DIRF,DIRCH)

! calculation of cerenkov angle from photon hit on detector and part. track
! resi_sin: linear approximation

      REAL R0(3)                ! point of photon emission (mid point on track)
      REAL DIR(3)               ! particle direction (unit vector)
      REAL RH(3)                ! photon hit on detector
      REAL RC(3), R_MI          ! center of curvature of mirror

      REAL DIRF(3)              ! result (photon direction vector in HERA-B) if
      REAL DIRCH(3)             ! result (photon direction vector in DIR) if
      LOGICAL OK                ! OK = .true.

      REAL R1(3), R2(3), RR(3), R1N, R2N, RRN
      REAL A, B, ALFA, TH, FI
      REAL C1, C2, S_ALFA

c     print*,'f:',r0,dir,rh,rc,r_mi
      CERENKOV_KOT=0
      DO I=1,3
         R1(I)=R0(I)-RC(I)
         R2(I)=RH(I)-RC(I)
      ENDDO
      R1N=SQRT(R1(1)**2+R1(2)**2+R1(3)**2)	
      R2N=SQRT(R2(1)**2+R2(2)**2+R2(3)**2)	
      A=R1N/R_MI
      B=R2N/R1N
      ALFA=(R1(1)*R2(1)+R1(2)*R2(2)+R1(3)*R2(3))/R1N/R2N
      ALFA=MAX(MIN(ALFA,1.),-1.)
      ALFA=ACOS(ALFA)
      CALL RESI_SIN(A,B,ALFA,TH,FI,OK)
      IF(.NOT.OK) RETURN
      S_ALFA=SIN(ALFA)
      C1=R_MI/R1N*SIN(ALFA-FI)/S_ALFA
      C2=R_MI/R2N*SIN(FI)/S_ALFA
      DO I=1,3
         RR(I)=(C1-1.)*R1(I)+C2*R2(I)
      ENDDO
      RRN=SQRT(RR(1)**2+RR(2)**2+RR(3)**2)	
      DO I=1,3
         DIRF(I)=RR(I)/RRN
      ENDDO
      CALL TRANSFORM_FROM_FIXED(DIR,DIRF,DIRCH)
      CERENKOV_KOT=1
      
      RETURN
      END


      SUBROUTINE RESI_SIN(A,B,C,X,Y,OK)
      
! find solution of equations:	sin(x) = a*sin(x+y)
!				sin(x+y)=b*sin(c+x-y)
! approximative solution (second order)

      REAL A,B,C                ! coeficients
      REAL X,Y                  ! result
      LOGICAL OK                ! if all OK
      
      ABB=1.-2*A*B+B
      IF(ABB.EQ.0) THEN
         OK=.FALSE.
         RETURN
      ENDIF

      X=A*B*C/ABB
      Y=(1.-A)*B*C/ABB

cc      DO I=1,2
         SX=SIN(X)
         CX=COS(X)
         SXY=SIN(X+Y)
         CXY=COS(X+Y)
         SCXY=SIN(C+X-Y)
         CCXY=COS(C+X-Y)
      
         A11=CX-A*CXY
         A12=-A*CXY
         B1=A*SXY-SX
         A21=CXY-B*CCXY
         A22=CXY+B*CCXY
         B2=B*SCXY-SXY
         
         D=A11*A22-A12*A21
         IF(D.EQ.0) THEN
            OK=.FALSE.
            RETURN
         ENDIF
      
         X=X+(B1*A22-B2*A12)/D
         Y=Y+(B2*A11-B1*A21)/D
cc      ENDDO

      OK=.TRUE.
      RETURN
      END
	  	

      SUBROUTINE TRANSFORM_FROM_FIXED(SMER_Z,VIN,VOUT)

! transformation of a vector from fixed system S to current system S'
! such that x' is in plane (x,z) -  horizontal in HERA-B
! SMER_Z() is unit vector of z-axis of S' given in fixed system S
! VIN()  are the coordinates of a vector in S
! VOUT() are the coordinates of a vector in S'

      REAL SMER_Z(3), VIN(3), VOUT(3)
      REAL SMER_X(3), SMER_Y(3)

      SS=1.-SMER_Z(2)**2
      IF(SS.LE.0) THEN
         VOUT(1)=VIN(1)
         VOUT(2)=VIN(3)*SMER_Z(2)
         VOUT(3)=-VIN(2)*SMER_Z(2)
      ELSE
         SS=SQRT(SS)
         SMER_X(1)=SMER_Z(3)/SS
         SMER_X(2)=0.
         SMER_X(3)=-SMER_Z(1)/SS
         CALL VEKT_PROD(SMER_Z,SMER_X,SMER_Y)
         CALL SKAL_PROD(VIN,SMER_X,VOUT(1))
         CALL SKAL_PROD(VIN,SMER_Y,VOUT(2))
         CALL SKAL_PROD(VIN,SMER_Z,VOUT(3))
      ENDIF

      RETURN
      END

 
      SUBROUTINE VEKT_PROD(A,B,C)

! vector product: C = A x B

      REAL A(3), B(3), C(3)

      C(1)=A(2)*B(3)-A(3)*B(2)
      C(2)=A(3)*B(1)-A(1)*B(3)
      C(3)=A(1)*B(2)-A(2)*B(1)

      RETURN
      END


      SUBROUTINE SKAL_PROD(A,B,S)

! scalar product: s=A.B

      REAL A(3), B(3), S

      S=A(1)*B(1)+A(2)*B(2)+A(3)*B(3)
      
      RETURN
      END

 

