#ifndef _riter_geometry_h_
#define _riter_geometry_h_

#include <arte/arte.hh>
#include <riter/RITER.hh>
#include <riter/RITERSupermodule.hh>
#include <riter/RITERPlanarmirror.hh>
#include <riter/RITERSphericalmirror.hh>
#include <riter/RITERBeamshroud.hh>

#define NUMROW 24
#define NUMCOL 12
#define NUMSM  32
#define NUMCH  16

#define NCHAN 86016   
#define NWCHA (NCHAN-1)/32+1 

/// Rough acceptance test for tracks
int IsInside(float tx,float ty);

/// Description of the RICH RITER geometry
class RITERGeometry : 
  public  RITERSphericalmirror, 
  public  RITERPlanarmirror,
  public  RITERSupermodule,  
  public  RITERBeamshroud  
{

private:
  ///  type=0-m16 =1-m4 sigma^2 =sig0^2+sigthc^2+sig_sfa^2 
  float sigma[2];   
public:
  /// error : ... , spherical abberation
  /// geometrical : $$sigma^2 =sig0^2+sigthc^2+sig_sfa^2$$  :
  /// sig0 =nn*sig_pad; nn={4-m1,2-m4, 1-m16}
  float sigthc, sig_sfa, sig_pad;   
  /// f_pad=f/d_pad 
  float f_pad;   
  /// N_0
  float an_zero;
  /// refractive index -average
  float ref_ind;  

  /// acceptance limits
  float  th_0,fi_0; 
  /// lens system acceptance
  float acc_lens;
  /// start and and of the rich
  float z_rich_min, z_rich_max; 
  /// square of the maximum and min  Ch.radius 
  float rmax2, rmin2;    
  float x_cut, y_cut, z_cut, z_magnet,p_min;
  int is_correction; 

  void   to_xyz(int ism,int ir,int ic,int nn,RITERVec &r,float &sig);  
  int src2det(int ism, int ir, int ic , int pmch , int type, 
	      int &sm, int &x0, int &y0  );

  int   Print();
  float Sigma(int type){ return sigma[type]; };
  int   Kuip();
  int   ReadCorrections();
  /// Correction due to missalignment of the spherical and planar mirrors
  float Correct_thc(int id_sfm,int id_plm,float th_cer,float fi_cer);
  int IsCorrections(){return is_correction; };
  RITERGeometry();
  ~RITERGeometry(){};
  
};

extern RITERGeometry *RITER_Geometry;

#endif




