#ifndef _riter_likelihood_h_
#define _riter_likelihood_h_

#include <riter/RITER.hh>

/// This class calculates the likelihood functions for different hypothesis
class RITERLikelihood {
  
private:
  /// number of sigma deviations type,m16/m4/small/big
  float window[4];        
  /// number of non-overlaping windows
  int    nval[8];         
  /// number of non-overlaping windows 
  float count[8];         
  /// minimums of non overlaping windows
  float min[8][2*NHIP];         
  /// maximums of non overlaping windows
  float max[8][2*NHIP];         
  /// likelihood value
  float lkh[NHIP];
  /// errors for different windows
  float sig[4];
  /// normalization 
  float A;
  float Nanti;
  /// expected Cherenkov angles
  float *thc;  
  /// expected number of signal photons for {..}
  float *stf;
  /// expected number of background
  float bgr;
  /// number of signal photon that come into likelihood
  float sfot;

public:
  /** 
    constructor of the class : 
    n ... number of hypothesis,
    thc ... cherenkov angles,  
    sigmas ... m16,m4
  */
  RITERLikelihood(int n,float *thc,float *stf,float sigm16, float sigm4);
  ~RITERLikelihood(){
  };
  /// Print the contents of the class
  int Print();
  /// test whether the value is in the interval type
  int IsInside(int type,float thc);
  int IsInside2(int type,float thc){  return IsInside(type+4,thc); };
  /// Fill different  counters with weighted photons + test windows
  int Fill(int type,float thc, float  weight); 
  /// Fill different  counters with weighted photons
  float Fill(int type,float  weight); 
  /// Fill log likelihood function with weighted photons
  int FillLKH(int type,float thc, float  weight); 
  int FillLKHtest(int type,float thc1,float *lkh);
  /// returns width as a function of a window_type
  float WindowWidth(int type){ return window[type]; };

  /// Initializes windows
  int SetWindow(int normalm16= 3  , int normalm4 =3,
		int small    = 3 ,  int big      = 9 );
  /// Concat intervals
  int Windows(int n, float *x,float *y);
  /// make squared windows
  int SqrWindows(int n);
    /// Subtract interval (a2,a3) from the interval (a0,a1)   
  int Subtract(float *a);
  /// Subtract two sets of intervals intervals A&(!B)
  int AntiWindows(int &n, float *x,float *y,
	      int n1, float *x1,float *y1);
  /// Returns value of the counter
  float  Number(int type){ return  count[type]; };
  /// Calculates integral \int_min_max Ax^3
  float Integral(int type);
  float GetNorm(){ return A;};
  /// returns the number of background photons
  float SetBackgroundPhotons();
  float BackgroundPhotons(){ return bgr; };
  /// returns the number of signal photons;
  float SignalPhotons(){ return sfot; };
  /// returns loglikelihood withouth poisonian part
  float Get(int i){ return lkh[i]; };
  /// calculates the background at angle x
  float Background(float x){ return A*x*x*x; };

};
//-------------------------------------------------------------
inline int  RITERLikelihood::IsInside(int type,float thc){
  // window type 0-m16 1-m4 2-small 3-big
  // cherenkov angle of the photon track pair
  for (int i=0 ; i<nval[type] ; i++ ){
    if ( (thc< max[type][i])&& (thc > min[type][i])) {
      return 1;      
    }
  }
  return 0;
}


//-------------------------------------------------------------

#endif


