#ifndef _riter_supermodule_h_
#define _riter_supermodule_h_
#include <riter/RITER.hh>
#include <riter/RITERPmt.hh>
#include <riter/RITERUtils.hh>
#include <rich/rich.h>

#define NUMROW 24
#define NUMCOL 12
#define NUMSM  32
#define NUMCH  16


class RITERSupermodule {
protected:
  RITERPmt  channel[NUMSM][NUMROW][NUMCOL][NUMCH];

  RITERVec  r0[14],vx[14],vy[14],vn[14];  // geometry of supermodules
  float       dvx[14],dvy[14],dxsm[14];
 
public:

  RITERSupermodule();
  ~RITERSupermodule(){};
  int PrintSupermodule();

  int IsInRange(int ism,int ir, int ic,int ich=0){
    // cout << "Is In Range " <<RITERVec(ism,ir,ic) << endl;
   if ( (ism>=NUMSM)    ||(ism<0)) return 0;
   if ( (ir >=NUMROW)   ||(ir<0 )) return 0;
   if ( (ic >=NUMCOL)   ||(ic<0 )) return 0;
   if ( (ich>=NUMCH)    ||(ich<0)) return 0;

   return 1;
  }

  int Type(int ism,int ir, int ic,int ich=0) {
    //  ^ ir              ism 0  1  2   3  4  5  6 
    //  |          
    //  o---> ic
    //  ic <---o          ism  13 12 11 10 9 8 7 
    //         |               
    //    ir   v
    
    if ( IsInRange(ism,ir,ic,ich) )
    
      return channel[ism][ir][ic][ich].Type(); 
    else return -1;
  };

  int Type(int ism,float r, float c) {
    //  ^ ir              ism 0  1  2   3  4  5  6 
    //  |          
    //  o---> ic
    //  ic <---o         ism  13 12 11  10 9  8  7 
    //         |               
    //    ir   v
    return Type(ism,(int)(r/4),(int)(c/4)); 
  };



  RITERVec Position(int ism,int ir, int ic,int ich=0) {
    return channel[ism][ir][ic][ich].Position(); 
  };
  RITERVec Position(int ism,float r, float c) {
    // r,c m16 pad units
    //  ^ r              ism 0  1  2   3  4  5  6 
    //  |          
    //  o---> c
    //  c <---o         ism   13 12 11 10 9 8 7 
    //        |              
    //    r   v

    int ir=(int) r;
    int ic=(int) c;
    int ppr=ir % 4;
    int ppc=ic % 4;
    ir/=4;
    ic/=4; 
    int type=channel[ism][ir][ic][0].Type();
    if (type <0) return RITERVec();
    int ich= 0;
    if (type) 
      ich= 3-ppc/2 -2*(ppr/2);  // m4
    else
      ich= 15-ppc -4*ppr;        // m16
    return Position(ism,ir, ic,ich);
  };
  
  int Set(RITERVec r,int type,int ism,int ir, int ic,int ich=0) {
    //  ^ ir              ism 0  1  2   3  4  5  6 
    //  |          
    //  o---> ic
    //  ic <---o         ism   13 12 11 10 9 8 7 
    //         |               
    //    ir   v

    return channel[ism][ir][ic][ich].Set(r,type);
  };
  float DistanceSM(int k) { return dxsm[k];}
  int IsDetector(RITERVec &r,
			   float  &alfa,float  &beta,int ism);
  int DetectorIntersection(RITERVec &r,
			   float  &alfa,float  &beta,int &ism);
  int DetectorIntersection(RITERVec &r,RITERVec &dir,int kje,
			   RITERVec &r1,float  &alfa,float  &beta,int &ism);
};

#endif










