#ifndef _RITER_HEP_THREEVECTOR_H
#define _RITER_HEP_THREEVECTOR_H

typedef int RITERBoolean;

#ifndef false
#define false 0
#endif
#ifndef true
#define true 1
#endif



#include <iostream.h>




class RITERVec {

public:

  enum { X=0, Y=1, Z=2, NUM_COORDINATES=3, SIZE=NUM_COORDINATES };
  // Safe indexing of the coordinates when using with matrices, arrays, etc.
  // (BaBar)

  inline RITERVec(float x = 0.0, float y = 0.0, float z = 0.0);
  // The constructor.

  inline RITERVec(const RITERVec &);
  // The copy constructor.

  inline ~RITERVec();
  // The destructor.

  float operator () (int) const;
  inline float operator [] (int) const;
  // Get components by index (Geant4).

  float & operator () (int);
  inline float & operator [] (int);
  // Set components by index.

  inline float x() const;
  inline float y() const;
  inline float z() const;
  // The components in cartesian coordinate system.

  inline void setX(float);
  inline void setY(float);
  inline void setZ(float);
  // Set the components in cartesian coordinate system.

  inline float phi() const;
  // The azimuth angle.

  inline float theta() const;
  // The polar angle.

  inline float cosTheta() const;
  // Cosine of the polar angle.

  inline float mag2() const;
  // The magnitude squared (rho^2 in spherical coordinate system).

  inline float mag() const;
  // The magnitude (rho in spherical coordinate system).

  inline void setPhi(float);
  // Set phi keeping mag and theta constant (BaBar).

  inline void setTheta(float);
  // Set theta keeping mag and phi constant (BaBar).

  inline void setMag(float);
  // Set magnitude keeping theta and phi constant (BaBar).

  inline float perp2() const;
  // The transverse component squared (R^2 in cylindrical coordinate system).

  inline float perp() const;
  // The transverse component (R in cylindrical coordinate system).

  inline void setPerp(float);
  // Set the transverse component keeping phi and z constant.

  inline float perp2(const RITERVec &) const;
  // The transverse component w.r.t. given axis squared.

  inline float perp(const RITERVec &) const;
  // The transverse component w.r.t. given axis.

  inline RITERVec & operator = (const RITERVec &);
  // Assignment.

  inline RITERBoolean operator == (const RITERVec &) const;
  inline RITERBoolean operator != (const RITERVec &) const;
  // Comparisons (Geant4). 

  inline RITERVec & operator += (const RITERVec &);
  // Addition.

  inline RITERVec & operator -= (const RITERVec &);
  // Subtraction.

  inline RITERVec operator - () const;
  // Unary minus.

  inline RITERVec & operator *= (float);
  // Scaling with real numbers.

  inline RITERVec unit() const;
  // Unit vector parallel to this.

  inline RITERVec orthogonal() const;
  // Vector orthogonal to this (Geant4).

  inline float dot(const RITERVec &) const;
  // Scalar product.

  inline RITERVec cross(const RITERVec &) const;
  // Cross product.

  inline float angle(const RITERVec &) const;
  // The angle w.r.t. another 3-vector.

  float pseudoRapidity() const;
  // Returns the pseudo-rapidity, i.e. -ln(tan(theta/2))

  void rotateX(float);
  // Rotates the RITERVec around the x-axis.

  void rotateY(float);
  // Rotates the RITERVec around the y-axis.

  void rotateZ(float);
  // Rotates the RITERVec around the z-axis.

  void rotateUz(const RITERVec&);
  // Rotates reference frame from Uz to newUz (unit vector) (Geant4).

  void rotate(float, const RITERVec &);
  // Rotates around the axis specified by another RITERVec.

  //  RITERVec & operator *= (const HepRotation &);
  //  RITERVec & transform(const HepRotation &);
  // Transformation with a Rotation matrix.

private:

  float dx, dy, dz;
  // The components.
};

std::ostream & operator << (std::ostream &out , const RITERVec & x);

std::istream & operator >> (std::istream &, RITERVec &);
// Input from a stream.

#include "RITERThreeVector.icc"

#endif /* _RITER_RITER_HEP_THREEVECTOR_H */

