#ifndef _riter_tracks_h_
#define _riter_tracks_h_
#include <riter/RITER.hh>
#include <arte/arte.hh>
#include <riter/RITERGeometry.hh>
#include <riter/RITERUtils.hh>
#include <riter/RITERLikelihood.hh>

/// RITER track   
class RITERTracks { 
private:
  RITERLikelihood *likelihood;
public:

  /// entrance point, direction
  RITERVec r,dir;       
  /// momentum
  float p ;               

  /// likelihoods for NHIP hypothesis
  float plkh[NHIP];
  /// measured number of signal photons  
  float sfot;
  /// expected number of signal photons for {..}
  float stf[NHIP];         
  // expected number of bgr photons 
  //float bgr;          
  /// portion of m16 hits for {..}
  float acgm16[NHIP];   
  /// geometrical acceptance for  {..}        
  float acg[NHIP];
  /// pointer to the geometry 
  RITERGeometry *geo;
  /// track error
  float sig;
  /// particle type (unknown=0,e=1,mu=2,pi=3,k=4,p=5)
  int  itype;            
  /// mean photon emission point
  RITERVec  r0;        
  /// particle hit on (up, down) sph. mirror 
  RITERVec  tr1[2];   
  /// direction after reflection (up, down)  
  RITERVec  tdir1[2]; 
  /// ring centers (up and down detector)
  RITERVec  trc[2]  ;  
  /// which mirror was hit =1 if particle hits upper sph. mirror
  short  wm ; 
  /// track path          
  float tpath;
  /// which planar mirror
  short id_plm[2] ;
  /// which spherical mirror
  short id_sfm[2];
   
  /// expected cerenkov angles for {0,e,mu,pi,k,p,e+e-} 
  float thc[NHIP];      
  /// expected min cerenkov angles for {...} 
  float thmin[2][NHIP];      
  /// expected max cerenkov angles for {..}  
  float thmax[2][NHIP];      
  ///  sigmas  for {0,e,mu,pi,k,p,e+e-} 
  float sig_mean[NHIP];     
 
  /// pointer to RTRA
  ArtePointer<RTRA> rtra;
  /// calculate track parameters
  int Calculate();
  /// calculate acceptance
  float  Acceptance(float, float *);
  /// calculate expected photons
  int    ExpectedPhotons(float thc,float path,
			 float &stfs, float &acg,float &acg16);
  float  acc_the(float theta);
  /// calculate expected background photons
  float  ExpectedBackground(int i,float *thc,float *sig,float *acg);
  /// print the contents of the class
  int    Print();
  /// expected cherenkov angle of the track with momentum p 
  float  CherenkovAngle(float p,int itype,float ref_ind); 
  /// pointer to  RITERLikelihood class
  RITERLikelihood *Likelihood(); 
  int CalculateLKH();
  int Test(int );

  /// constructor 
  RITERTracks(const RITERVec &rtrack, const RITERVec &dirtrack,  
	      float momentum,   	      float sigma,  
              int   type=0, 
	      ArtePointer<RTRA> artetrack = ArtePointer<RTRA>() );
  /// destructor
  ~RITERTracks(){ if (likelihood) delete likelihood; };
};


#endif




